@@ -12,21 +12,26 @@ import re
 import webbrowser
 import uuid
 
-from dataclasses import dataclass
-import hydra
-from hydra.core.config_store import ConfigStore
-from omegaconf import DictConfig, OmegaConf
-from flatten_dict import flatten
+try:
+    from dataclasses import dataclass
+    import hydra
+    from hydra.core.config_store import ConfigStore
+    from omegaconf import DictConfig, OmegaConf
+    from flatten_dict import flatten
 
-import azureml
-from azureml.core import Datastore
-from azureml.core import Experiment
-from azureml.core import Dataset
-from azureml.pipeline.core import PipelineRun
-from azure.ml.component._core._component_definition import (
-    ComponentDefinition,
-    ComponentType,
-)
+    import azureml
+    from azureml.core import Datastore
+    from azureml.core import Experiment
+    from azureml.core import Dataset
+    from azureml.pipeline.core import PipelineRun
+    from azure.ml.component._core._component_definition import (
+        ComponentDefinition,
+        ComponentType,
+    )
+except ImportError as error:
+    raise ImportError(
+        f"{error.msg}. Please install using `pip install shrike[pipeline]`."
+    )
 
 from shrike import __version__
 from shrike.pipeline.aml_connect import azureml_connect, current_workspace
@@ -265,7 +270,7 @@ class AMLPipelineHelper:
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
+            module_instance (Module): the AML module we need to add settings to
             mpi (bool): is job mpi ?
             target (str): force target compute over hydra conf
             node_count (int): force node_count over hydra conf
@@ -336,11 +341,11 @@ class AMLPipelineHelper:
         output_mode=None,
         **custom_runtime_arguments,
     ):
-        """Applies settings for a hdi module.
+        """Applies settings for a HDI module.
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
+            module_instance (Module): the AML module we need to add settings to
             target (str): force target compute over hydra conf
             driver_cores (int): force driver_cores over hydra conf
             driver_memory (str): force driver_memory over hydra conf
@@ -353,7 +358,7 @@ class AMLPipelineHelper:
             custom_runtime_arguments (dict): any additional custom args
         """
         print(
-            f"Using hdi compute target {self.config.compute.hdi_prod_target} to run {module_name} from pipeline class {self.__class__.__name__}"
+            f"Using HDI compute target {self.config.compute.hdi_prod_target} to run {module_name} from pipeline class {self.__class__.__name__}"
         )
         if custom_runtime_arguments:
             print(f"Adding custom runtime arguments {custom_runtime_arguments}")
@@ -417,6 +422,7 @@ class AMLPipelineHelper:
         mini_batch_size=None,
         run_invocation_timeout=None,
         run_max_try=None,
+        error_threshold=None,
         input_mode=None,
         output_mode=None,
         **custom_runtime_arguments,
@@ -425,15 +431,19 @@ class AMLPipelineHelper:
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
-            windows (bool): is the module using windows compute?
-            gpu (bool): is the module using gpu compute?
+            module_instance (Module): the AML module we need to add settings to
+            windows (bool): is the module using Windows compute?
+            gpu (bool): is the module using GPU compute?
             target (str): force target compute over hydra conf
             node_count (int): force node_count over hydra conf
             process_count_per_node (int): force process_count_per_node over hydra conf
             mini_batch_size (int): force mini_batch_size over hydra conf
             run_invocation_timeout (int): force run_invocation_timeout over hydra conf
             run_max_try (int): force run_max_try over hydra conf
+            error_threshold (int): The number of file failures for the input FileDataset that should be ignored during processing.
+                If the error count goes above this value, then the job will be aborted.
+                Error threshold is for the entire input and not for individual mini-batches sent to run() method.
+                The range is [-1, int.max]. -1 indicates ignoring all failures during processing.
             input_mode (str): force input_mode over hydra conf
             output_mode (str): force output_mode over hydra conf
             custom_runtime_arguments (dict): any additional custom args
@@ -442,7 +452,7 @@ class AMLPipelineHelper:
             if windows:
                 if gpu:
                     raise ValueError(
-                        "A gpu compute target with Windows OS is not available yet!"
+                        "A GPU compute target with Windows OS is not available yet!"
                     )
                 else:
                     _target = self.config.compute.windows_cpu_dc_target
@@ -455,7 +465,7 @@ class AMLPipelineHelper:
             if windows:
                 if gpu:
                     raise ValueError(
-                        "A gpu compute target with Windows OS is not available yet!"
+                        "A GPU compute target with Windows OS is not available yet!"
                     )
                 else:
                     _target = self.config.compute.windows_cpu_prod_target
@@ -481,7 +491,11 @@ class AMLPipelineHelper:
             else self.config.compute.parallel_node_count,
             process_count_per_node=process_count_per_node
             if process_count_per_node is not None
-            else self.config.compute.parallel_process_count_per_node,
+            else (
+                self.config.compute.parallel_process_count_per_node
+                if "parallel_process_count_per_node" in self.config.compute
+                else None
+            ),
             mini_batch_size=str(
                 mini_batch_size
                 if mini_batch_size is not None
@@ -493,6 +507,9 @@ class AMLPipelineHelper:
             run_max_try=run_max_try
             if run_max_try is not None
             else self.config.compute.parallel_run_max_try,
+            error_threshold=error_threshold
+            if error_threshold is not None
+            else self.config.compute.parallel_error_threshold,
             **custom_runtime_arguments,
         )
 
@@ -518,9 +535,9 @@ class AMLPipelineHelper:
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
+            module_instance (Module): the AML module we need to add settings to
             mpi (bool): is the job mpi?
-            gpu (bool): is the job using gpu?
+            gpu (bool): is the job using GPU?
             target (str): force target compute over hydra conf
             node_count (int): force node_count over hydra conf
             process_count_per_node (int): force process_count_per_node over hydra conf
@@ -612,7 +629,7 @@ class AMLPipelineHelper:
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
+            module_instance (Module): the AML module we need to add settings to
             input_mode (str): force input_mode over hydra conf
             output_mode (str): force output_mode over hydra conf
             scope_param (str): Parameters to pass to scope e.g. Nebula parameters, VC allocation parameters etc.
@@ -640,16 +657,18 @@ class AMLPipelineHelper:
         self,
         module_name,
         module_instance,
+        compliant=True,
         target=None,
         input_mode=None,
         output_mode=None,
         **custom_runtime_arguments,
     ):
-        """Applies settings for a hdi module.
+        """Applies settings for a HDI module.
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
+            module_instance (Module): the AML module we need to add settings to
+            compliant (bool): destination datastore, `compliant_datastore` if True, else `noncompliant_datastore`
             target (str): force target compute over hydra conf
             input_mode (str): force input_mode over hydra conf
             output_mode (str): force output_mode over hydra conf
@@ -669,7 +688,140 @@ class AMLPipelineHelper:
 
         if input_mode:
             self._set_all_inputs_to(module_instance, input_mode)
-        self._set_all_outputs_to(module_instance, output_mode)
+        self._set_all_outputs_to(module_instance, output_mode, compliant)
+
+    def _apply_sweep_runsettings(
+        self,
+        module_name,
+        module_instance,
+        windows=False,
+        gpu=False,
+        target=None,
+        input_mode=None,
+        output_mode=None,
+        algorithm=None,
+        primary_metric=None,
+        goal=None,
+        policy_type=None,
+        evaluation_interval=None,
+        delay_evaluation=None,
+        slack_factor=None,
+        slack_amount=None,
+        truncation_percentage=None,
+        max_total_trials=None,
+        max_concurrent_trials=None,
+        timeout_minutes=None,
+        **custom_runtime_arguments,
+    ):
+        """Applies settings for a sweep component.
+
+        Args:
+            module_name (str): name of the module from the module manifest (required_modules() method)
+            module_instance (Module): the AML module we need to add settings to
+            target (str): force target compute over hydra conf
+            input_mode (str): force input_mode over hydra conf
+            output_mode (str): force output_mode over hydra conf
+
+            # For below sweep specific parameters configurations, see below doc link for more info:
+            # https://componentsdk.azurewebsites.net/components/sweep_component.html#set-runsettings
+            algorithm (str): sweep sampling method
+            primary_metric (str): the primary metric of the hyperparameter tuning to optimize
+            goal (str): whether the primary metric will be maximize or minimize when evaluating the trials
+            policy_type (str): sweep early termination policy type
+            evaluation_interval (int): frequency of applying the policy
+            delay_evaluation (int): delays the first policy evaluation for a specified number of intervals
+            slack_factor (float): the slack allowed with respect to the best performing training run, as a ratio
+            slack_amount (float): the slack allowed with respect to the best performing training run, as an absolute ampunt. You should only specify either slack_factor or slack_amount, but not both.
+            truncation_percentage (int): the percentage of lowest performing runs to terminate at each evaluation interval. An integer value between 1 and 99.
+            max_total_trials (int): maximum number of trial runs. Must be an integer between 1 and 1000.
+            max_concurrent_trials (int): maximum number of runs that can run concurrently. If not specified, all runs launch in parallel. If specified, must be an integer between 1 and 100.
+            timeout_minutes (int): maximum duration, in minutes, of the hyperparameter tuning experiment. Runs after this duration are canceled.
+
+            custom_runtime_arguments (dict): any additional custom args
+        """
+        if target is not None:
+            target = target
+        elif self.module_loader.is_local(module_name):
+            if windows:
+                if gpu:
+                    raise ValueError(
+                        "A GPU compute target with Windows OS is not available yet!"
+                    )
+                else:
+                    target = self.config.compute.windows_cpu_dc_target
+            else:
+                if gpu:
+                    target = self.config.compute.linux_gpu_dc_target
+                else:
+                    target = self.config.compute.linux_cpu_dc_target
+        else:
+            if windows:
+                if gpu:
+                    raise ValueError(
+                        "A GPU compute target with Windows OS is not available yet!"
+                    )
+                else:
+                    target = self.config.compute.windows_cpu_prod_target
+            else:
+                if gpu:
+                    target = self.config.compute.linux_gpu_prod_target
+                else:
+                    target = self.config.compute.linux_cpu_prod_target
+        print(
+            f"Using target {target} to run sweep component {module_name} from pipeline class {self.__class__.__name__}"
+        )
+        if custom_runtime_arguments:
+            print(f"Adding custom runtime arguments {custom_runtime_arguments}")
+
+        if input_mode:
+            self._set_all_inputs_to(module_instance, input_mode)
+        if output_mode:
+            self._set_all_outputs_to(module_instance, output_mode)
+
+        if algorithm:
+            module_instance.runsettings.sweep.algorithm = algorithm
+        if primary_metric:
+            module_instance.runsettings.sweep.objective.configure(
+                primary_metric=primary_metric
+            )
+        if goal:
+            module_instance.runsettings.sweep.objective.configure(goal=goal)
+        if policy_type:
+            module_instance.runsettings.sweep.early_termination.configure(
+                policy_type=policy_type
+            )
+        if evaluation_interval:
+            module_instance.runsettings.sweep.early_termination.configure(
+                evaluation_interval=evaluation_interval
+            )
+        if delay_evaluation:
+            module_instance.runsettings.sweep.early_termination.configure(
+                delay_evaluation=delay_evaluation
+            )
+        if slack_factor:
+            module_instance.runsettings.sweep.early_termination.configure(
+                slack_factor=slack_factor
+            )
+        if slack_amount:
+            module_instance.runsettings.sweep.early_termination.configure(
+                slack_amount=slack_amount
+            )
+        if truncation_percentage:
+            module_instance.runsettings.sweep.early_termination.configure(
+                truncation_percentage=truncation_percentage
+            )
+        if max_total_trials:
+            module_instance.runsettings.sweep.limits.configure(
+                max_total_trials=max_total_trials
+            )
+        if max_concurrent_trials:
+            module_instance.runsettings.sweep.limits.configure(
+                max_concurrent_trials=max_concurrent_trials
+            )
+        if timeout_minutes:
+            module_instance.runsettings.sweep.limits.configure(
+                timeout_minutes=timeout_minutes
+            )
 
     def _check_module_runsettings_consistency(self, module_key, module_instance):
         """Verifies if entry at module_key matches the module instance description"""
@@ -692,6 +844,70 @@ class AMLPipelineHelper:
                 f"During build() of graph class {self.__class__.__name__}, call to self.apply_recommended_runsettings() is wrong: key used as first argument ('{module_key}') maps to a module reference {module_manifest_entry} which name is different from the module instance provided as 2nd argument (name={module_instance.name}), did you use the wrong module key as first argument?"
             )
 
+    def _get_component_name_from_instance(self, component_instance):
+        component_manifest_list = self.config.modules.manifest
+        component_name = component_instance.name
+        for component_manifest_entry in component_manifest_list:
+            try:
+                if component_manifest_entry["name"] == component_name:
+                    return component_manifest_entry["key"] or component_name
+                if "namespace" in component_manifest_entry:
+                    component_entry_name = (
+                        component_manifest_entry["namespace"]
+                        + "://"
+                        + component_manifest_entry["name"]
+                    )
+                    if component_entry_name == component_name:
+                        return component_manifest_entry["key"] or component_name
+            except:
+                pass
+        raise ValueError(
+            f"Could not find component matching {component_name}. Please check your spelling."
+        )
+
+    def apply_smart_runsettings(
+        self,
+        component_instance,
+        gpu=False,  # can't autodetect that
+        hdi="auto",
+        windows="auto",
+        parallel="auto",
+        mpi="auto",
+        scope="auto",
+        datatransfer="auto",
+        sweep="auto",
+        **custom_runtime_arguments,
+    ):
+        """Applies regular settings for a given component.
+
+        Args:
+            component_instance (Component): the AML component we need to add settings to
+            gpu (bool): is the component using GPU?
+            hdi (bool): is the component using HDI/Spark?
+            windows (bool): is the component using Windows compute?
+            parallel (bool): is the component using ParallelRunStep?
+            mpi (bool): is the component using Mpi?
+            scope (bool): is the component using scope?
+            datatransfer (bool): is the component using datatransfer?
+            sweep (bool): is the component using sweep?
+            custom_runtime_arguments (dict): any additional custom args
+        """
+        # infer component_name
+        component_name = self._get_component_name_from_instance(component_instance)
+        self.apply_recommended_runsettings(
+            component_name,
+            component_instance,
+            gpu,
+            hdi,
+            windows,
+            parallel,
+            mpi,
+            scope,
+            datatransfer,
+            sweep,
+            **custom_runtime_arguments,
+        )
+
     def apply_recommended_runsettings(
         self,
         module_name,
@@ -703,16 +919,17 @@ class AMLPipelineHelper:
         mpi="auto",
         scope="auto",
         datatransfer="auto",
+        sweep="auto",
         **custom_runtime_arguments,
     ):
         """Applies regular settings for a given module.
 
         Args:
             module_name (str): name of the module from the module manifest (required_modules() method)
-            module_instance (Module): the aml module we need to add settings to
-            gpu (bool): is the module using gpu?
-            hdi (bool): is the module using hdi/spark?
-            windows (bool): is the module using windows compute?
+            module_instance (Module): the AML module we need to add settings to
+            gpu (bool): is the module using GPU?
+            hdi (bool): is the module using HDI/Spark?
+            windows (bool): is the module using Windows compute?
             parallel (bool): is the module using ParallelRunStep?
             mpi (bool): is the module using Mpi?
             custom_runtime_arguments (dict): any additional custom args
@@ -741,13 +958,19 @@ class AMLPipelineHelper:
             if scope:
                 print(f"Module {module_name} detected as SCOPE: {scope}")
 
+        if sweep == "auto":
+            sweep = str(module_instance.type) == "SweepComponent"
+            if sweep:
+                print(f"Module {module_name} detected as SweepComponent: {sweep}")
+
         if windows == "auto":
             if (
                 str(module_instance.type) == "HDInsightComponent"
                 or str(module_instance.type) == "ScopeComponent"
                 or str(module_instance.type) == "DataTransferComponent"
+                or str(module_instance.type) == "SweepComponent"
             ):
-                # hdi/scope/datatransfer modules might not have that environment object
+                # HDI/scope/datatransfer/sweep modules might not have that environment object
                 windows = False
             else:
                 windows = (
@@ -771,6 +994,16 @@ class AMLPipelineHelper:
             )
             return
 
+        if sweep:
+            self._apply_sweep_runsettings(
+                module_name,
+                module_instance,
+                windows=windows,
+                gpu=gpu,
+                **custom_runtime_arguments,
+            )
+            return
+
         if windows:
             # no detonation chamber, we an't use "use_local" here
             self._apply_windows_runsettings(
